 /********************************************************************/
 /*                                                                  */
 /* Program name: DLQMON                                             */
 /*                                                                  */
 /* Description: C program that acts as a DLQ monitor                */
 /*                                                                  */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*                                                                  */
 /*   DLQMON is a C program that acts as a dead letter queue         */
 /*   monitor reads the dead letter queue and tells you if anything  */
 /*   appears on it. It leaves it there until you remove it.         */
 /*                                                                  */
 /*   It's second parameter is the queue manager name, the first is  */
 /*   the dead letter queue name. If these are not supplied it uses  */
 /*   the defaults.                                                  */
 /*                                                                  */
 /********************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <stdio.h>

 #ifdef OS2
   /******************************************************************/
   /* for beep                                                       */
   /******************************************************************/
   #define INCL_DOSPROCESS
   #include <os2.h>
 #endif

 /********************************************************************/
 /* includes for MQI                                                 */
 /********************************************************************/
 #include <cmqc.h>
 #include <cmqcfc.h>

 /********************************************************************/
 /* prototype the functions                                          */
 /********************************************************************/
 void printreas(MQLONG reason);

 /********************************************************************/
 /* global variable                                                  */
 /********************************************************************/
 MQCFH    *dlqmsg;                    /* dlqmsg message buffer       */

 /********************************************************************/
 /* the program itself                                               */
 /********************************************************************/
 int main(int argc, char **argv)
 {
   /******************************************************************/
   /* declare variables                                              */
   /******************************************************************/
   int  i;                             /* auxiliary counter          */
   /******************************************************************/
   /* Declare MQI structures needed                                  */
   /******************************************************************/
   MQOD     od = {MQOD_DEFAULT};      /* Object Descriptor           */
   MQMD     md = {MQMD_DEFAULT};      /* Message Descriptor          */
   MQGMO   gmo = {MQGMO_DEFAULT};     /* get message options         */
   /******************************************************************/
   /* note, uses defaults where it can                               */
   /******************************************************************/
   MQHCONN  Hcon;                      /* connection handle          */
   MQHOBJ   Hobj;                      /* object handle              */
   MQLONG   O_options;                 /* MQOPEN options             */
   MQLONG   C_options;                 /* MQCLOSE options            */
   MQLONG   CompCode;                  /* completion code            */
   MQLONG   OpenCode;                  /* MQOPEN completion code     */
   MQLONG   Reason;                    /* reason code                */
   MQLONG   CReason;                   /* reason code for MQCONN     */
   MQLONG   buflen;                    /* buffer length              */
   MQLONG   dlqmsglen;                 /* message length received    */
   MQCHAR   command[1100];             /* call command string ...    */
   MQCHAR   p1[600];                   /* ApplId insert              */
   MQCHAR   p2[900];                   /* dlqmsg insert              */
   MQCHAR   p3[600];                   /* Environment insert         */
   MQLONG   mytype;                    /* saved application type     */
   char     QMName[50];                /* queue manager name         */
   MQCFST  *paras;                     /* the parameters             */
   int      counter;                   /* loop counter               */
   time_t   ltime;                     /* for printing the time      */
   char     strInfo[61];               /* output string              */
   char    dummy[11];                  /* Need for ltoa()            */
   MQHOBJ  hobj;                       /* inquire Object handle      */
   MQOD    odenq       = { MQOD_DEFAULT };  /* Obj descr for enquire */
   MQLONG  opts;                       /* MQOPEN/MQCLOSE options     */
   MQLONG  selectcnt   =  1;           /* Count of inquire selectors */
   MQLONG  selectors[1];               /* Array of attr selectors    */
   MQLONG  intattrcnt  =  0;           /* Count of integer attrs     */
   MQLONG  intattr[1];                 /* Array of integer attrs     */

   /******************************************************************/
   /* union to help read the header                                  */
   /******************************************************************/
   union UDLH
   {
      MQDLH   mdlh;
      MQCHAR  cdlh[sizeof(MQDLH)];
   } dlh;

   /******************************************************************/
   /* See whether they want help                                     */
   /******************************************************************/
   if (argc > 1 && argv[1][0] == '?')
   {
     printf("Usage: dlqmon [DLQ] [QMgrName]\n" );
     exit(0);
   }

   /******************************************************************/
   /* Connect to queue manager                                       */
   /* If a queue manager name is specified then use it, otherwise    */
   /* use the default one                                            */
   /******************************************************************/
   QMName[0] = 0;                      /* default queue manager      */
   if (argc > 2)
     strcpy(QMName, argv[2]);
   MQCONN(QMName,                      /* queue manager              */
          &Hcon,                       /* connection handle          */
          &CompCode,                   /* completion code            */
          &CReason);                   /* reason code                */

   /******************************************************************/
   /* report reason and stop if it failed                            */
   /******************************************************************/
   if (CompCode == MQCC_FAILED)
   {
     printf("MQCONN ended with reason code %ld\n", CReason);
     printreas(CReason);
     exit(CReason);
   }

   /******************************************************************/
   /* Find the name of the dead letter queue                         */
   /******************************************************************/
   {
     /****************************************************************/
     /* Open the queue manager for inquire                           */
     /****************************************************************/
     odenq.ObjectType = MQOT_Q_MGR;
     strcpy(odenq.ObjectQMgrName,QMName);
     opts = MQOO_INQUIRE | MQOO_FAIL_IF_QUIESCING;

     MQOPEN(Hcon,&odenq,opts,&hobj,&CompCode,&Reason);

     /****************************************************************/
     /* if it failed then explain why and exit                       */
     /****************************************************************/
     if(CompCode == MQCC_FAILED)
     {
        printf("Unable to open queue manager (%s) for inquire\n",
                QMName);
        printreas(Reason);
        MQDISC(&Hcon,&CompCode,&Reason);
        exit(Reason);
     }

     /****************************************************************/
     /* Find out the name of the queue manager                       */
     /****************************************************************/
     selectors[0] = MQCA_DEAD_LETTER_Q_NAME;
     MQINQ(Hcon,hobj,selectcnt,selectors,intattrcnt,intattr,
            sizeof(od.ObjectName),od.ObjectName,&CompCode,&Reason);
     if(CompCode == MQCC_FAILED)
     {
        printf("Inquire failed to get dead letter queue name\n");
        printreas(Reason);
        MQCLOSE(Hcon,&hobj,MQCO_NONE,&CompCode,&Reason);
        MQDISC(&Hcon,&CompCode,&Reason);
        exit(Reason);
     }

     /****************************************************************/
     /* close the inquire queue                                      */
     /****************************************************************/
     C_options = 0;                  /* no close options             */
     MQCLOSE(Hcon,                   /* connection handle            */
            &hobj,                   /* object handle                */
            C_options,
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */
     /****************************************************************/
     /* report reason it failed to close, if any                     */
     /****************************************************************/
     if (Reason != MQRC_NONE)
     {
       printf("MQCLOSE ==> %ld\n", Reason);
     }
   }

   /******************************************************************/
   /* just in case they really wanted to monitor a different queue   */
   /* we'll put this bit in                                          */
   /******************************************************************/
   if (argc > 1)
     strcpy(od.ObjectName, argv[1]);

   /******************************************************************/
   /* Open the event queue for input; exclusive or shared.  Use of   */
   /* the queue is controlled by the queue definition here           */
   /******************************************************************/
   O_options = MQOO_INPUT_AS_Q_DEF     /* open queue for input       */
         + MQOO_FAIL_IF_QUIESCING      /* but not if MQM stopping    */
         + MQOO_BROWSE;
   MQOPEN(Hcon,                        /* connection handle          */
          &od,                         /* object descriptor for queue*/
          O_options,                   /* open options               */
          &Hobj,                       /* object handle              */
          &CompCode,                   /* completion code            */
          &Reason);                    /* reason code                */

   /******************************************************************/
   /* report reason, if any; stop if failed                          */
   /******************************************************************/
   if (Reason != MQRC_NONE)
   {
     printf("MQOPEN (%s) ==> %ld\n", od.ObjectName, Reason);
   }

   OpenCode = CompCode;                /* keep for conditional close */

   /******************************************************************/
   /*   Get messages from the message queue                          */
   /******************************************************************/
   while (CompCode != MQCC_FAILED)
   {
     /****************************************************************/
     /* We don't know how big this message is so just get the        */
     /* descriptor first                                             */
     /* wait for new messages, and when we find one lock it so that  */
     /* we can get it when we've got the storage                     */
     /* We use convert just in case this is running as a client      */
     /****************************************************************/
     gmo.Options = MQGMO_WAIT + MQGMO_BROWSE_NEXT + MQGMO_LOCK
                      + MQGMO_ACCEPT_TRUNCATED_MSG + MQGMO_CONVERT;

     buflen = 0;                       /* ammount of message to get  */
     /****************************************************************/
     /* we used to have this as unlimited wait but we can't do       */
     /* this because if it's done from a client and the client dies  */
     /* then the server doesn't notice and keeps the queue open      */
     /* so now we loop on a 5 second wait                            */
     /****************************************************************/
     /*gmo.WaitInterval = MQWI_UNLIMITED;   no time limit            */
     gmo.WaitInterval = 5000;          /* 5 seconds                  */

     /****************************************************************/
     /* clear selectors to get messages in sequence                  */
     /****************************************************************/
     memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
     memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

     /****************************************************************/
     /* wait for event message                                       */
     /* if one doesn't arrive then wait again                        */
     /****************************************************************/
     printf("...>\n");
     for (Reason = MQRC_NO_MSG_AVAILABLE,
          CompCode = MQCC_FAILED;
          Reason == MQRC_NO_MSG_AVAILABLE
                && CompCode == MQCC_FAILED; )
     {
       MQGET(Hcon,                     /* connection handle          */
             Hobj,                     /* object handle              */
             &md,                      /* message descriptor         */
             &gmo,                     /* get message options        */
             buflen,                   /* buffer length              */
             dlqmsg,                   /* dlqmsg message buffer      */
             &dlqmsglen,               /* message length             */
             &CompCode,                /* completion code            */
             &Reason);                 /* reason code                */
     }

     /****************************************************************/
     /* report reason, if any                                        */
     /****************************************************************/
     if (Reason != MQRC_NONE && Reason != MQRC_TRUNCATED_MSG_ACCEPTED)
     {
       printf("MQGET ==> %ld\n", Reason);
       printreas(Reason);
     }
     else
     {
       gmo.Options = MQGMO_NO_WAIT + MQGMO_BROWSE_MSG_UNDER_CURSOR
                        + MQGMO_CONVERT;
       buflen = dlqmsglen;             /* ammount of message to get  */
       dlqmsg = malloc(buflen);
       if (dlqmsg != NULL)
       {
         /************************************************************/
         /* clear selectors to get messages in sequence              */
         /************************************************************/
         memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
         memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

         /************************************************************/
         /* get the message                                          */
         /************************************************************/
         printf("...>\n");
         MQGET(Hcon,                   /* connection handle          */
               Hobj,                   /* object handle              */
               &md,                    /* message descriptor         */
               &gmo,                   /* get message options        */
               buflen,                 /* buffer length              */
               dlqmsg,                 /* dlqmsg message buffer      */
               &dlqmsglen,             /* message length             */
               &CompCode,              /* completion code            */
               &Reason);               /* reason code                */

         /************************************************************/
         /* report reason, if any                                    */
         /************************************************************/
         if (Reason != MQRC_NONE)
         {
           printf("MQGET ==> %ld\n", Reason);
           printreas(Reason);
         }
       }
       else
       {
         CompCode = MQCC_FAILED;
       }
     }
     /****************************************************************/
     /* . . . process each message received                          */
     /****************************************************************/
     if (CompCode != MQCC_FAILED)
     {
       /**************************************************************/
       /* announce a message                                         */
       /**************************************************************/
       #ifdef OS2
         {
           unsigned short tone;
           for (tone = 1; tone < 8000; tone = tone * 2)
           {
             DosBeep(tone,25);
           }
         }
       #else
         printf("\a\a\a\a\a\a\a");
       #endif
       time(&ltime);
       printf(ctime(&ltime));

       /**************************************************************/
       /* if the data length is wrong then something has gone        */
       /* badly wrong as we've only just checked it before we got    */
       /* the storage                                                */
       /**************************************************************/
       if (dlqmsglen != buflen)
         printf("DataLength = %ld?\n", dlqmsglen);
       else
       {
         /************************************************************/
         /* right let's look at the data                             */
         /* make sure it's got a dead letter header on it and then   */
         /* read it                                                  */
         /************************************************************/
         if (!strncmp(((MQDLH*)dlqmsg)->StrucId,MQDLH_STRUC_ID,
               sizeof(((MQDLH*)dlqmsg)->StrucId)))
         {

           memcpy(dlh.cdlh,dlqmsg,sizeof(MQDLH));
           printf("PutDate = %8.8s PutTime = %8.8s Reason = %ld\n"
                  "DestQName = %.48s\nDestQMgrName = %.48s\n",
                  dlh.mdlh.PutDate,
                  dlh.mdlh.PutTime,
                  dlh.mdlh.Reason,
                  dlh.mdlh.DestQName,
                  dlh.mdlh.DestQMgrName);

           /**********************************************************/
           /* print the reason again, but decoded                    */
           /**********************************************************/
           printreas(dlh.mdlh.Reason);

         }
         else
         {
           printf("msg has no DLQ header\n");
         }
       }
       /**************************************************************/
       /* free the storage we obtained for the message               */
       /**************************************************************/
       free(dlqmsg);
     }                                 /* end process for GET        */
   }                                   /* end msg processing loop    */

   /******************************************************************/
   /* close the dead letter queue - if it was opened                 */
   /******************************************************************/
   if (OpenCode != MQCC_FAILED)
   {
     C_options = 0;                  /* no close options             */
     MQCLOSE(Hcon,                   /* connection handle            */
            &Hobj,                   /* object handle                */
            C_options,
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */
     /****************************************************************/
     /* report reason, if any                                        */
     /****************************************************************/
     if (Reason != MQRC_NONE)
     {
       printf("MQCLOSE ==> %ld\n", Reason);
     }
   }

   /******************************************************************/
   /* Disconnect from MQM  (unless previously connected)             */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED)
   {
     MQDISC(&Hcon,                   /* connection handle            */
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */
     /****************************************************************/
     /* report reason, if any                                        */
     /****************************************************************/
     if (Reason != MQRC_NONE)
     {
       printf("MQDISC ended with reason code %ld\n", Reason);
     }
   }

 /********************************************************************/
 /*                                                                  */
 /* END OF DLQMON                                                    */
 /*                                                                  */
 /********************************************************************/
 }

/*********************************************************************/
/* define macros                                                     */
/*********************************************************************/
 #define PRINTREAS(param)                                            \
    case param:                                                      \
      printf("Reason = %s\n",#param);                                \
      break;

void printreas(MQLONG reason)
{
  switch (reason)
  {
    PRINTREAS(MQRCCF_CFH_TYPE_ERROR)
    PRINTREAS(MQRCCF_CFH_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFH_VERSION_ERROR)
    PRINTREAS(MQRCCF_CFH_MSG_SEQ_NUMBER_ERR)
    PRINTREAS(MQRCCF_CFH_CONTROL_ERROR)
    PRINTREAS(MQRCCF_CFH_PARM_COUNT_ERROR)
    PRINTREAS(MQRCCF_CFH_COMMAND_ERROR)
    PRINTREAS(MQRCCF_COMMAND_FAILED)
    PRINTREAS(MQRCCF_CFIN_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFST_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFST_STRING_LENGTH_ERR)
    PRINTREAS(MQRCCF_FORCE_VALUE_ERROR)
    PRINTREAS(MQRCCF_STRUCTURE_TYPE_ERROR)
    PRINTREAS(MQRCCF_CFIN_PARM_ID_ERROR)
    PRINTREAS(MQRCCF_CFST_PARM_ID_ERROR)
    PRINTREAS(MQRCCF_MSG_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFIN_DUPLICATE_PARM)
    PRINTREAS(MQRCCF_CFST_DUPLICATE_PARM)
    PRINTREAS(MQRCCF_PARM_COUNT_TOO_SMALL)
    PRINTREAS(MQRCCF_PARM_COUNT_TOO_BIG)
    PRINTREAS(MQRCCF_Q_ALREADY_IN_CELL)
    PRINTREAS(MQRCCF_Q_TYPE_ERROR)
    PRINTREAS(MQRCCF_MD_FORMAT_ERROR)
    PRINTREAS(MQRCCF_REPLACE_VALUE_ERROR)
    PRINTREAS(MQRCCF_CFIL_DUPLICATE_VALUE)
    PRINTREAS(MQRCCF_CFIL_COUNT_ERROR)
    PRINTREAS(MQRCCF_CFIL_LENGTH_ERROR)
    PRINTREAS(MQRCCF_QUIESCE_VALUE_ERROR)
    PRINTREAS(MQRCCF_MSG_SEQ_NUMBER_ERROR)
    PRINTREAS(MQRCCF_PING_DATA_COUNT_ERROR)
    PRINTREAS(MQRCCF_PING_DATA_COMPARE_ERROR)
    PRINTREAS(MQRCCF_CHANNEL_TYPE_ERROR)
    PRINTREAS(MQRCCF_PARM_SEQUENCE_ERROR)
    PRINTREAS(MQRCCF_XMIT_PROTOCOL_TYPE_ERR)
    PRINTREAS(MQRCCF_BATCH_SIZE_ERROR)
    PRINTREAS(MQRCCF_DISC_INT_ERROR)
    PRINTREAS(MQRCCF_SHORT_RETRY_ERROR)
    PRINTREAS(MQRCCF_SHORT_TIMER_ERROR)
    PRINTREAS(MQRCCF_LONG_RETRY_ERROR)
    PRINTREAS(MQRCCF_LONG_TIMER_ERROR)
    PRINTREAS(MQRCCF_SEQ_NUMBER_WRAP_ERROR)
    PRINTREAS(MQRCCF_MAX_MSG_LENGTH_ERROR)
    PRINTREAS(MQRCCF_PUT_AUTH_ERROR)
    PRINTREAS(MQRCCF_PURGE_VALUE_ERROR)
    PRINTREAS(MQRCCF_CFIL_PARM_ID_ERROR)
    PRINTREAS(MQRCCF_MSG_TRUNCATED)
    PRINTREAS(MQRCCF_CCSID_ERROR)
    PRINTREAS(MQRCCF_ENCODING_ERROR)
    PRINTREAS(MQRCCF_DATA_CONV_VALUE_ERROR)
    PRINTREAS(MQRCCF_INDOUBT_VALUE_ERROR)
    PRINTREAS(MQRCCF_ESCAPE_TYPE_ERROR)
    PRINTREAS(MQRCCF_CHANNEL_TABLE_ERROR)
    PRINTREAS(MQRCCF_MCA_TYPE_ERROR)
    PRINTREAS(MQRCCF_CHL_INST_TYPE_ERROR)
    PRINTREAS(MQRCCF_CHL_STATUS_NOT_FOUND)
    PRINTREAS(MQRCCF_OBJECT_ALREADY_EXISTS)
    PRINTREAS(MQRCCF_OBJECT_WRONG_TYPE)
    PRINTREAS(MQRCCF_LIKE_OBJECT_WRONG_TYPE)
    PRINTREAS(MQRCCF_OBJECT_OPEN)
    PRINTREAS(MQRCCF_ATTR_VALUE_ERROR)
    PRINTREAS(MQRCCF_UNKNOWN_Q_MGR)
    PRINTREAS(MQRCCF_Q_WRONG_TYPE)
    PRINTREAS(MQRCCF_OBJECT_NAME_ERROR)
    PRINTREAS(MQRCCF_ALLOCATE_FAILED)
    PRINTREAS(MQRCCF_HOST_NOT_AVAILABLE)
    PRINTREAS(MQRCCF_CONFIGURATION_ERROR)
    PRINTREAS(MQRCCF_CONNECTION_REFUSED)
    PRINTREAS(MQRCCF_ENTRY_ERROR)
    PRINTREAS(MQRCCF_SEND_FAILED)
    PRINTREAS(MQRCCF_RECEIVED_DATA_ERROR)
    PRINTREAS(MQRCCF_RECEIVE_FAILED)
    PRINTREAS(MQRCCF_CONNECTION_CLOSED)
    PRINTREAS(MQRCCF_NO_STORAGE)
    PRINTREAS(MQRCCF_NO_COMMS_MANAGER)
    PRINTREAS(MQRCCF_LISTENER_NOT_STARTED)
    PRINTREAS(MQRCCF_BIND_FAILED)
    PRINTREAS(MQRCCF_CHANNEL_INDOUBT)
    PRINTREAS(MQRCCF_MQCONN_FAILED)
    PRINTREAS(MQRCCF_MQOPEN_FAILED)
    PRINTREAS(MQRCCF_MQGET_FAILED)
    PRINTREAS(MQRCCF_MQPUT_FAILED)
    PRINTREAS(MQRCCF_PING_ERROR)
    PRINTREAS(MQRCCF_CHANNEL_IN_USE)
    PRINTREAS(MQRCCF_CHANNEL_NOT_FOUND)
    PRINTREAS(MQRCCF_UNKNOWN_REMOTE_CHANNEL)
    PRINTREAS(MQRCCF_REMOTE_QM_UNAVAILABLE)
    PRINTREAS(MQRCCF_REMOTE_QM_TERMINATING)
    PRINTREAS(MQRCCF_MQINQ_FAILED)
    PRINTREAS(MQRCCF_NOT_XMIT_Q)
    PRINTREAS(MQRCCF_CHANNEL_DISABLED)
    PRINTREAS(MQRCCF_USER_EXIT_NOT_AVAILABLE)
    PRINTREAS(MQRCCF_COMMIT_FAILED)
    PRINTREAS(MQRCCF_CHANNEL_ALREADY_EXISTS)
    PRINTREAS(MQRCCF_DATA_TOO_LARGE)
    PRINTREAS(MQRCCF_CHANNEL_NAME_ERROR)
    PRINTREAS(MQRCCF_XMIT_Q_NAME_ERROR)
    PRINTREAS(MQRCCF_MCA_NAME_ERROR)
    PRINTREAS(MQRCCF_SEND_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_SEC_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_MSG_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_RCV_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_XMIT_Q_NAME_WRONG_TYPE)
    PRINTREAS(MQRCCF_MCA_NAME_WRONG_TYPE)
    PRINTREAS(MQRCCF_DISC_INT_WRONG_TYPE)
    PRINTREAS(MQRCCF_SHORT_RETRY_WRONG_TYPE)
    PRINTREAS(MQRCCF_SHORT_TIMER_WRONG_TYPE)
    PRINTREAS(MQRCCF_LONG_RETRY_WRONG_TYPE)
    PRINTREAS(MQRCCF_LONG_TIMER_WRONG_TYPE)
    PRINTREAS(MQRCCF_PUT_AUTH_WRONG_TYPE)
    PRINTREAS(MQRCCF_MISSING_CONN_NAME)
    PRINTREAS(MQRCCF_CONN_NAME_ERROR)
    PRINTREAS(MQRCCF_MQSET_FAILED)
    PRINTREAS(MQRCCF_CHANNEL_NOT_ACTIVE)
    PRINTREAS(MQRCCF_TERMINATED_BY_SEC_EXIT)
    PRINTREAS(MQRCCF_DYNAMIC_Q_SCOPE_ERROR)
    PRINTREAS(MQRCCF_CELL_DIR_NOT_AVAILABLE)
    PRINTREAS(MQRC_NONE)
    PRINTREAS(MQRC_ALIAS_BASE_Q_TYPE_ERROR)
    PRINTREAS(MQRC_ALREADY_CONNECTED)
    PRINTREAS(MQRC_BACKED_OUT)
    PRINTREAS(MQRC_BUFFER_ERROR)
    PRINTREAS(MQRC_BUFFER_LENGTH_ERROR)
    PRINTREAS(MQRC_CHAR_ATTR_LENGTH_ERROR)
    PRINTREAS(MQRC_CHAR_ATTRS_ERROR)
    PRINTREAS(MQRC_CHAR_ATTRS_TOO_SHORT)
    PRINTREAS(MQRC_CONNECTION_BROKEN)
    PRINTREAS(MQRC_DATA_LENGTH_ERROR)
    PRINTREAS(MQRC_DYNAMIC_Q_NAME_ERROR)
    PRINTREAS(MQRC_ENVIRONMENT_ERROR)
    PRINTREAS(MQRC_EXPIRY_ERROR)
    PRINTREAS(MQRC_FEEDBACK_ERROR)
    PRINTREAS(MQRC_GET_INHIBITED)
    PRINTREAS(MQRC_HANDLE_NOT_AVAILABLE)
    PRINTREAS(MQRC_HCONN_ERROR)
    PRINTREAS(MQRC_HOBJ_ERROR)
    PRINTREAS(MQRC_INHIBIT_VALUE_ERROR)
    PRINTREAS(MQRC_INT_ATTR_COUNT_ERROR)
    PRINTREAS(MQRC_INT_ATTR_COUNT_TOO_SMALL)
    PRINTREAS(MQRC_INT_ATTRS_ARRAY_ERROR)
    PRINTREAS(MQRC_SYNCPOINT_LIMIT_REACHED)
    PRINTREAS(MQRC_MAX_CONNS_LIMIT_REACHED)
    PRINTREAS(MQRC_MD_ERROR)
    PRINTREAS(MQRC_MISSING_REPLY_TO_Q)
    PRINTREAS(MQRC_MSG_TYPE_ERROR)
    PRINTREAS(MQRC_MSG_TOO_BIG_FOR_Q)
    PRINTREAS(MQRC_MSG_TOO_BIG_FOR_Q_MGR)
    PRINTREAS(MQRC_NO_MSG_AVAILABLE)
    PRINTREAS(MQRC_NO_MSG_UNDER_CURSOR)
    PRINTREAS(MQRC_NOT_AUTHORIZED)
    PRINTREAS(MQRC_NOT_OPEN_FOR_BROWSE)
    PRINTREAS(MQRC_NOT_OPEN_FOR_INPUT)
    PRINTREAS(MQRC_NOT_OPEN_FOR_INQUIRE)
    PRINTREAS(MQRC_NOT_OPEN_FOR_OUTPUT)
    PRINTREAS(MQRC_NOT_OPEN_FOR_SET)
    PRINTREAS(MQRC_OBJECT_CHANGED)
    PRINTREAS(MQRC_OBJECT_IN_USE)
    PRINTREAS(MQRC_OBJECT_TYPE_ERROR)
    PRINTREAS(MQRC_OD_ERROR)
    PRINTREAS(MQRC_OPTION_NOT_VALID_FOR_TYPE)
    PRINTREAS(MQRC_OPTIONS_ERROR)
    PRINTREAS(MQRC_PERSISTENCE_ERROR)
    PRINTREAS(MQRC_PERSISTENT_NOT_ALLOWED)
    PRINTREAS(MQRC_PRIORITY_EXCEEDS_MAXIMUM)
    PRINTREAS(MQRC_PRIORITY_ERROR)
    PRINTREAS(MQRC_PUT_INHIBITED)
    PRINTREAS(MQRC_Q_DELETED)
    PRINTREAS(MQRC_Q_FULL)
    PRINTREAS(MQRC_Q_NOT_EMPTY)
    PRINTREAS(MQRC_Q_SPACE_NOT_AVAILABLE)
    PRINTREAS(MQRC_Q_TYPE_ERROR)
    PRINTREAS(MQRC_Q_MGR_NAME_ERROR)
    PRINTREAS(MQRC_Q_MGR_NOT_AVAILABLE)
    PRINTREAS(MQRC_REPORT_OPTIONS_ERROR)
    PRINTREAS(MQRC_SECOND_MARK_NOT_ALLOWED)
    PRINTREAS(MQRC_SECURITY_ERROR)
    PRINTREAS(MQRC_SELECTOR_COUNT_ERROR)
    PRINTREAS(MQRC_SELECTOR_LIMIT_EXCEEDED)
    PRINTREAS(MQRC_SELECTOR_ERROR)
    PRINTREAS(MQRC_SELECTOR_NOT_FOR_TYPE)
    PRINTREAS(MQRC_SIGNAL_OUTSTANDING)
    PRINTREAS(MQRC_SIGNAL_REQUEST_ACCEPTED)
    PRINTREAS(MQRC_STORAGE_NOT_AVAILABLE)
    PRINTREAS(MQRC_SYNCPOINT_NOT_AVAILABLE)
    PRINTREAS(MQRC_TRIGGER_CONTROL_ERROR)
    PRINTREAS(MQRC_TRIGGER_DEPTH_ERROR)
    PRINTREAS(MQRC_TRIGGER_MSG_PRIORITY_ERR)
    PRINTREAS(MQRC_TRIGGER_TYPE_ERROR)
    PRINTREAS(MQRC_TRUNCATED_MSG_ACCEPTED)
    PRINTREAS(MQRC_TRUNCATED_MSG_FAILED)
    PRINTREAS(MQRC_UNKNOWN_ALIAS_BASE_Q)
    PRINTREAS(MQRC_UNKNOWN_OBJECT_NAME)
    PRINTREAS(MQRC_UNKNOWN_OBJECT_Q_MGR)
    PRINTREAS(MQRC_UNKNOWN_REMOTE_Q_MGR)
    PRINTREAS(MQRC_WAIT_INTERVAL_ERROR)
    PRINTREAS(MQRC_XMIT_Q_TYPE_ERROR)
    PRINTREAS(MQRC_XMIT_Q_USAGE_ERROR)
    PRINTREAS(MQRC_NOT_OPEN_FOR_PASS_ALL)
    PRINTREAS(MQRC_NOT_OPEN_FOR_PASS_IDENT)
    PRINTREAS(MQRC_NOT_OPEN_FOR_SET_ALL)
    PRINTREAS(MQRC_NOT_OPEN_FOR_SET_IDENT)
    PRINTREAS(MQRC_CONTEXT_HANDLE_ERROR)
    PRINTREAS(MQRC_CONTEXT_NOT_AVAILABLE)
    PRINTREAS(MQRC_SIGNAL1_ERROR)
    PRINTREAS(MQRC_OBJECT_ALREADY_EXISTS)
    PRINTREAS(MQRC_OBJECT_DAMAGED)
    PRINTREAS(MQRC_RESOURCE_PROBLEM)
    PRINTREAS(MQRC_ANOTHER_Q_MGR_CONNECTED)
    PRINTREAS(MQRC_UNKNOWN_REPORT_OPTION)
    PRINTREAS(MQRC_SUPPRESSED_BY_EXIT)
    PRINTREAS(MQRC_FORMAT_ERROR)
    PRINTREAS(MQRC_SOURCE_CCSID_ERROR)
    PRINTREAS(MQRC_SOURCE_INTEGER_ENC_ERROR)
    PRINTREAS(MQRC_SOURCE_DECIMAL_ENC_ERROR)
    PRINTREAS(MQRC_SOURCE_FLOAT_ENC_ERROR)
    PRINTREAS(MQRC_TARGET_CCSID_ERROR)
    PRINTREAS(MQRC_TARGET_INTEGER_ENC_ERROR)
    PRINTREAS(MQRC_TARGET_DECIMAL_ENC_ERROR)
    PRINTREAS(MQRC_TARGET_FLOAT_ENC_ERROR)
    PRINTREAS(MQRC_NOT_CONVERTED)
    PRINTREAS(MQRC_CONVERTED_MSG_TOO_BIG)
    PRINTREAS(MQRC_ADAPTER_STORAGE_SHORTAGE)
    PRINTREAS(MQRC_ADAPTER_CONN_LOAD_ERROR)
    PRINTREAS(MQRC_ADAPTER_SERV_LOAD_ERROR)
    PRINTREAS(MQRC_ADAPTER_DEFS_ERROR)
    PRINTREAS(MQRC_ADAPTER_DEFS_LOAD_ERROR)
    PRINTREAS(MQRC_ADAPTER_DISC_LOAD_ERROR)
    PRINTREAS(MQRC_CICS_WAIT_FAILED)
    PRINTREAS(MQRC_SOURCE_LENGTH_ERROR)
    PRINTREAS(MQRC_TARGET_LENGTH_ERROR)
    PRINTREAS(MQRC_SOURCE_BUFFER_ERROR)
    PRINTREAS(MQRC_TARGET_BUFFER_ERROR)
    PRINTREAS(MQRC_DBCS_ERROR)
    PRINTREAS(MQRC_TRUNCATED)
    PRINTREAS(MQRC_ASID_MISMATCH)
    PRINTREAS(MQRC_CONN_ID_IN_USE)
    PRINTREAS(MQRC_Q_MGR_QUIESCING)
    PRINTREAS(MQRC_Q_MGR_STOPPING)
    PRINTREAS(MQRC_DUPLICATE_RECOV_COORD)
    PRINTREAS(MQRC_PMO_ERROR)
    PRINTREAS(MQRC_API_EXIT_NOT_FOUND)
    PRINTREAS(MQRC_API_EXIT_LOAD_ERROR)
    PRINTREAS(MQRC_REMOTE_Q_NAME_ERROR)
    PRINTREAS(MQRC_GMO_ERROR)
    PRINTREAS(MQRC_PAGESET_FULL)
    PRINTREAS(MQRC_PAGESET_ERROR)
    PRINTREAS(MQRC_NAME_NOT_VALID_FOR_TYPE)
    PRINTREAS(MQRC_UNEXPECTED_ERROR)
    PRINTREAS(MQRC_UNKNOWN_XMIT_Q)
    PRINTREAS(MQRC_UNKNOWN_DEF_XMIT_Q)
    PRINTREAS(MQRC_DEF_XMIT_Q_TYPE_ERROR)
    PRINTREAS(MQRC_DEF_XMIT_Q_USAGE_ERROR)
    PRINTREAS(MQRC_NAME_IN_USE)
    PRINTREAS(MQRC_CONNECTION_QUIESCING)
    PRINTREAS(MQRC_CONNECTION_STOPPING)
    PRINTREAS(MQRC_ADAPTER_NOT_AVAILABLE)
    PRINTREAS(MQRC_MSG_ID_ERROR)
    PRINTREAS(MQRC_CORREL_ID_ERROR)
    PRINTREAS(MQRC_FILE_SYSTEM_ERROR)
    PRINTREAS(MQRC_NO_MSG_LOCKED)
    PRINTREAS(MQRC_CONNECTION_NOT_AUTHORIZED)
    PRINTREAS(MQRC_MSG_TOO_BIG_FOR_CHANNEL)
    PRINTREAS(MQRC_CALL_IN_PROGRESS)
    PRINTREAS(MQRC_Q_MGR_ACTIVE)
    PRINTREAS(MQRC_Q_MGR_NOT_ACTIVE)
    PRINTREAS(MQRC_Q_DEPTH_HIGH)
    PRINTREAS(MQRC_Q_DEPTH_LOW)
    PRINTREAS(MQRC_Q_SERVICE_INTERVAL_HIGH)
    PRINTREAS(MQRC_Q_SERVICE_INTERVAL_OK)
    PRINTREAS(MQRC_HCONFIG_ERROR)
    PRINTREAS(MQRC_FUNCTION_ERROR)
    PRINTREAS(MQRC_CHANNEL_STARTED)
    PRINTREAS(MQRC_CHANNEL_STOPPED)
    PRINTREAS(MQRC_CHANNEL_CONV_ERROR)
    PRINTREAS(MQRC_SERVICE_NOT_AVAILABLE)
    PRINTREAS(MQRC_INITIALIZATION_FAILED)
    PRINTREAS(MQRC_TERMINATION_FAILED)
    PRINTREAS(MQRC_UNKNOWN_Q_NAME)
    PRINTREAS(MQRC_SERVICE_ERROR)
    PRINTREAS(MQRC_Q_ALREADY_EXISTS)
    PRINTREAS(MQRC_USER_ID_NOT_AVAILABLE)
    PRINTREAS(MQRC_UNKNOWN_ENTITY)
    PRINTREAS(MQRC_UNKNOWN_AUTH_ENTITY)
    PRINTREAS(MQRC_UNKNOWN_REF_OBJECT)
    PRINTREAS(MQFB_SYSTEM_FIRST)
    PRINTREAS(MQFB_EXPIRATION)
    PRINTREAS(MQFB_COA)
    PRINTREAS(MQFB_COD)
    PRINTREAS(MQFB_QUIT)
    PRINTREAS(MQFB_CHANNEL_COMPLETED)
    PRINTREAS(MQFB_CHANNEL_FAIL_RETRY)
    PRINTREAS(MQFB_CHANNEL_FAIL)
    PRINTREAS(MQFB_APPL_CANNOT_BE_STARTED)
    PRINTREAS(MQFB_TM_ERROR)
    PRINTREAS(MQFB_APPL_TYPE_ERROR)
    PRINTREAS(MQFB_STOPPED_BY_MSG_EXIT)
    PRINTREAS(MQFB_XMIT_Q_MSG_ERROR)
    PRINTREAS(MQFB_SYSTEM_LAST)
    PRINTREAS(MQFB_APPL_FIRST)
    PRINTREAS(MQFB_APPL_LAST)
    default:
      printf("It's an unknown reason, %ld\n",
              reason);
      break;
  }
}
